###
### "THE BEER-WARE LICENSE":
### Alberto Ramos wrote this file. As long as you retain this 
### notice you can do whatever you want with this stuff. If we meet some 
### day, and you think this stuff is worth it, you can buy me a beer in 
### return. <alberto.ramos@cern.ch>
###
### file:    DataTypes.jl
### created: Mon Nov 22 14:51:00 2021
###                               

struct BIData{N,T}
    x::NTuple{N,T}
    y::NTuple{N,T}
    dy::NTuple{N,T}
end
import Base.show
function Base.show(io::IO, d::BIData{N,T}) where {N,T}

    println(io, "## Data (type: ", T, ")")
    for i in 1:N
        Printf.@printf(io, "  %8.4f     %8.4f +/- %6.2e\n", d.x[i], d.y[i], d.dy[i])
    end
    println(io, "## END Data")

    return nothing
end

abstract type Prior end

struct GaussPrior{N,T} <: Prior
    mu::NTuple{N,T}
    ds::NTuple{N,T}
end
function (s::GaussPrior{N,T})(x) where {N,T}

    fv = ( (x[1]-s.mu[1])/s.ds[1] )^2/2 + log(s.ds[1])
    for i in 2:N
        fv = fv + ( (x[i]-s.mu[i])/s.ds[i] )^2/2 + log(s.ds[i])
    end

    return fv
end
function Base.show(io::IO, d::GaussPrior{N,T}) where {N,T}

    println(io, "## Gassian Prior (type: ", T, ")")
    print(io, " # Mean:  ")
    for i in 1:N
        Printf.@printf(io, "  %8.4f", d.mu[i])
        if i%5 == 0
            print(io, "\n          ")
        end
    end
    print(io, "\n")
    print(io, " # Width: ")
    for i in 1:N
        Printf.@printf(io, "  %8.2e", d.ds[i])
        if i%5 == 0
            print(io, "\n          ")
        end
    end
    println(io, "\n## END Prior")

    return nothing
end

