###
### "THE BEER-WARE LICENSE":
### Alberto Ramos wrote this file. As long as you retain this 
### notice you can do whatever you want with this stuff. If we meet some 
### day, and you think this stuff is worth it, you can buy me a beer in 
### return. <alberto.ramos@cern.ch>
###
### file:    Models.jl
### created: Mon Nov 22 10:54:49 2021
###                               

abstract type Model end

mutable struct ModelSeries{T1,T2,T3} <: Model
    ip::Vector{T1}
    fc::Vector{T2}
    fp::Vector{T3}
end
ModelSeries(ip;coeffs=Vector{Float64}(),powers=Vector{Int64}()) = ModelSeries(ip,coeffs,powers)

mutable struct ModelPade <: Model
    nn::Int64
    nd::Int64
end

nparameters(s::ModelPade)   = s.nn+s.nd+1
nparameters(s::ModelSeries) = length(s.ip)

function (s::ModelSeries)(x, p)

    if (length(p) == 0) && (length(s.fc) == 0)
        return 0.0
    end
    
    if (length(p) > 0)
        res = p[1]*x^(s.ip[1])
        for k in 2:length(p)
            res = res + p[k]*x^(s.ip[k])
        end
    else
        res = zero(eltype(s.fc))
    end

    if (length(s.fc) > 0)
        res = res + s.fc[1]*x^(s.fp[1])
        for k in 2:length(s.fc)
            res = res + s.fc[k]*x^(s.fp[k])
        end
    end
    
    return res
end

function (s::ModelPade)(x, prm)

    if (s.nn == 0) && (s.nd == 0)
        return 0.0
    end

    if s.nn == 0
        vn = prm[1]
    elseif s.nn > 0
        vn = prm[s.nn+1]*x + prm[s.nn]
        for k in s.nn-1:-1:1
            vn = vn*x + prm[k]
        end
    else
        return 0.0
    end

    if s.nd == 1
        vd = 1.0 + prm[s.nn+2]*x
    elseif s.nd > 1
        vd = prm[s.nn+1+s.nd]*x + prm[s.nn+s.nd]
        for k in s.nd-2:-1:1
            vd = vd*x + prm[s.nn+1+k]
        end
        vd = vd*x + 1.0
    else
        return vn
    end

    return vn/vd
end

struct ModelNN{F} <: Model
    n::Int64
    f::F
end

nparameters(s::ModelNN) = 3*s.n+1
function (s::ModelNN)(x,p)

    fv = p[1] * s.f(x*p[s.n+1] + p[2*s.n+1]) + p[end]
    for i in 2:s.n
        fv = fv + p[i] * s.f(x*p[s.n+i] + p[2*s.n+i])
    end

    return fv
end
