###
### "THE BEER-WARE LICENSE":
### Alberto Ramos wrote this file. As long as you retain this 
### notice you can do whatever you want with this stuff. If we meet some 
### day, and you think this stuff is worth it, you can buy me a beer in 
### return. <alberto.ramos@cern.ch>
###
### file:    testhmc.jl
### created: Mon Nov 22 15:12:09 2021
###                               

using BayesianInference

# Data generation
np = 20
xv = rand(np)
yv = .-xv.^3 .+ xv.^2 .+ 3 .* xv .+ 2 .+ 0.1 .* randn(np)
dy = [0.1 for n in 1:np]
dt = BIData(tuple(xv...), tuple(yv...), tuple(dy...))
println(dt)

# Model and Priors
#mdl = ModelNN(20, x -> 1.0/(1.0 + exp(-x)))
mdl = ModelSeries([0,1,2,3])
pr = GaussPrior(tuple(zeros(nparameters(mdl))...), ntuple(i->100.0, nparameters(mdl)))
println(pr)

# MD integrator and WorkSpace
intr = omf4(Float64, 0.07, 15)
ws = BIWorkSpace(mdl,dt,pr)

# First go to the minima (max. probability)
pth = zeros(nparameters(mdl))
for i in 1:10
    relax!(pth, 10, mdl, dt, pr, ws; log=true)
end

# Thermalization
nth = 100
for i in 1:nth
    dh, acc = HMC!(pth, intr, mdl, dt, pr, ws)
    println("THM $i/$nth: ", acc, " ", dh)
end

# Simulation
nt = 10000
sm = zeros(nparameters(mdl))
@time let nacc = 0
    for i in 1:nt
        dh, acc = HMC!(pth, intr, mdl, dt, pr, ws)
        if acc
            nacc = nacc + 1
        end
        println("MSM $i/$nt: ", acc, " ", dh, "    ", pth)
        sm .= sm .+ pth
    end
    println("# Acceptance ($nt traj.): ", 100.0*nacc/nt, "%")
end

# Results
println("Results: ", sm./nt)
