
using BayesianInference, Test, Zygote

approx(a,b;tol=1.0E-12) = abs(a-b) < tol

poly = ModelSeries([0,1,2,3])
pade = ModelPade(3,0)

p = randn(4)
x = rand()

a = poly(x, p)
b = pade(x, p)
println("Model Series: ", a)
println("Model Pade:   ", b)
@test approx(a,b)

den = ModelSeries([0,1,2])
pade = ModelPade(3,2)

p = randn(7)
p[5] = 1.0
pp = [p[1],p[2],p[3],p[4],p[6],p[7]]
x = rand()

a1 = poly(x, @view p[1:4])
a2 = den(x, @view p[5:7])
a = a1/a2
b = pade(x, pp)
println("Model Series: ", a)
println("Model Pade:   ", b)
@test approx(a,b)

np = 20
NN = ModelNN(np, x -> 1.0/(1.0 + exp(-x)))
p = tuple(randn(3*np+1)...)
println("Function value: ", NN(x, p))

# Testing derivative
h = 1.0E-7
D1 = (NN(x+h,p)-NN(x-h,p))/(2*h)
D2 = gradient(x -> NN(x,p), x)[1]
@test approx(D1,D2,tol=1.0E-6)

#frc = gradient(p -> NN(x,p), p)[1]
#println(typeof(p))
#println(typeof(frc))

